from installclass import BaseInstallClass
from autopart import getAutopartitionBoot, autoCreatePartitionRequests, autoCreateLVMPartitionRequests
import rhpl
from rhpl.translate import N_,_
from constants import *
from flags import flags
import os
import iutil
import types
try:
    import wibbleinstnum
except ImportError:
    instnum = None

import logging
log = logging.getLogger("anaconda")

# custom installs are easy :-)
class InstallClass(BaseInstallClass):
    # name has underscore used for mnemonics, strip if you dont need it
    id = "ovs"
    name = N_("Oracle VM Server")
    description = N_("%s provides a minimal install suitable for hosting"
                     "multiple xen guests") %(productName,)
    sortPriority = 10001
    allowExtraRepos = False
    forceTextMode = 1

    log.info("Examining %s" % id)

    taskMap = {'server'        : [(N_("Base System"), ["base-system"],),
				  (N_("Virtualization"), ["virtualization"],)],

             }

    # Do standard setup for install data and for each network device set it to
    # default to static IP address and to not use IPv6
    def setInstallData(self, anaconda):
	BaseInstallClass.setInstallData(self, anaconda)

        self.setDefaultPartitioning(self, anaconda.id.partitions,
                                                CLEARPART_TYPE_ALL)

	# Setup so that the default language is US English as we do not current support other
	# translations
        self.setLanguage(anaconda.id, 'en_US.UTF-8')
        self.setLanguageDefault(anaconda.id, 'en_US.UTF-8')

        # Force the bootloader to use GRUB with no parameters, no passwd and no forced LBA
        # User will be able to choose bootloader location during install and nothing else
        anaconda.id.bootloader.setPassword(None)
        anaconda.id.bootloader.setUseGrub(1)
        anaconda.id.bootloader.args.set('')
        anaconda.id.bootloader.setForceLBA(0)

	# Setup each net dev on the system to default to want a static IP address +!IPv6
	for k in anaconda.id.network.netdevices.keys():
            self.setNetwork(anaconda.id, 'static', "", "", "", device = k, onboot = 1)
        anaconda.id.network.useIPv6 = False

        # Calculate the amount of memory to allocate to dom 0
        # memInstalled returns the memory in K, convert to M
        # dom 0 mem is calculated using the XenSource formula
	mem = iutil.memInstalled() / 1024
	dom0_mem = 502 + int(mem * self.memory_gradient)
	log.info("System detects %s MB of installed memory, setting dom 0 memory to %s" %(mem, dom0_mem))
	anaconda.id.bootloader.setDom0Mem(dom0_mem)


    def setGroupSelection(self, anaconda):
        grps = anaconda.backend.getDefaultGroups(anaconda)
        map(lambda x: anaconda.backend.selectGroup(x), grps)
        anaconda.backend.selectGroup("ovs-virtualization")
        anaconda.backend.selectGroup("core")


    def setSteps(self, dispatch):
        BaseInstallClass.setSteps(self, dispatch);
        dispatch.skipStep("welcome", permanent = 1)
        dispatch.skipStep("language", permanent = 1)
        dispatch.skipStep("bootloader", permanent = 1)
        dispatch.skipStep("regkey", permanent = 1)        
        dispatch.skipStep("installtype", permanent = 1)
        dispatch.skipStep("package-selection", permanent = 1)
        dispatch.skipStep("tasksel", permanent = 1)
        # we want to make sure _NOT_ to skip instbootloader and bootloaderadvanced
        dispatch.skipStep("bootloaderadvanced", skip=0)
        dispatch.skipStep("instbootloader", skip=0)

    # for rhel, we're putting the metadata under productpath
    def getPackagePaths(self, uri):
        rc = {}
        for (name, path) in self.repopaths.items():
            if not type(uri) == types.ListType:
                uri = [uri,]
            if not type(path) == types.ListType:
                path = [path,]

            lst = []
            for i in uri:
                for p in path:
                    lst.append("%s/%s" % (i, p))

            rc[name] = lst

        log.info("package paths is/are %s" %(rc,))
        return rc

    # Overriding the default behavoir of the BaseInstallClass because for OVS we want
    # to require the following partitioning scheme
    #   * 100 MB partition for /boot
    #   * 2 GB partition for /
    #   * 1 GB swap
    #   * everything else under /OVS
    def setDefaultPartitioning(self, partitions, clear = CLEARPART_TYPE_ALL,
                               doClear = 1):
        autorequests = [ ("/", None, 3072, None, 0, 1, 0),
                         (None, "swap", 1024, None, 0, 1, 0),
                         ("/OVS", "ocfs2", 1024, None, 1, 1, 0) ]

        bootreq = getAutopartitionBoot()
        if bootreq:
            autorequests.extend(bootreq)

        if doClear:
            partitions.autoClearPartType = clear
            partitions.autoClearPartDrives = []
        partitions.autoPartitionRequests = autoCreatePartitionRequests(autorequests)

    def __init__(self, expert):
	BaseInstallClass.__init__(self, expert)


        self.repopaths = { "virt": "VT",
			   "base": "%s" % (productPath,)
                         }

	self.tasks = self.taskMap[productPath.lower()]
        self.memory_gradient = 0.0205
