#
# ovs_agent_test.py: text mode OVS portal configuration dialog
#
# Kurt Hackel <kurt.hackel@oracle.com>
# Kevin Lyons <kevin.x.lyons@oracle.com>
#
# Copyright 2007 Oracle
#
# This software may be freely redistributed under the terms of the GNU
# library public license.
#
# You should have received a copy of the GNU Library Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import iutil
import os
import isys
import string
import network
import sha
from snack import *
from constants_text import *
from constants import *
from rhpl.translate import _

descr = { 'ovsagent':   'OVS Agent Password' }

def has_bad_chars(pw):
    allowed = string.digits + string.ascii_letters + string.punctuation + " "
    for letter in pw:
        if letter not in allowed:
            return 1
    return 0

def handleShortPasswords(screen):
    ButtonChoiceWindow(screen, _("Password Length"),
                       _("The root password must be at least 6 characters "
                         "long."), buttons = [ TEXT_OK_BUTTON ], width = 50)

def handleMismatchingPasswords(screen):
    ButtonChoiceWindow(screen, _("Password Mismatch"),
                       _("The passwords you entered were different. Please "
                         "try again."),buttons = [ TEXT_OK_BUTTON ], width = 50)

def handleBadCharacters(screen):
    ButtonChoiceWindow(screen, _("Error with Password"),
                       _("Requested password contains non-ascii characters "
                         "which are not allowed for use in password."),
                         buttons = [ TEXT_OK_BUTTON ], width = 50)

class OVSAgentWindow:
    def __call__(self, screen, anaconda, showonboot = 1):
        toplevel = GridFormHelp (screen, _("Oracle VM Agent password"),
                                 "ovsagent", 1, 3)

        toplevel.add (TextboxReflowed(37, _("Enter a password for the Oracle VM "
                "agent (ovs-agent). This password is used in Oracle VM manager to "
                "manage and monitor this server and its guest VMs. "
				"You must type it twice to ensure you know "
				"what it is and didn't make a mistake "
				"in typing.")), 0, 0, (0, 0, 0, 1))

        thegrid = Grid(2, 2)

        entry1 = Entry (24, password = 1)
        entry2 = Entry (24, password = 1)

        # if it's set already, use that... otherwise, make them enter it
        if anaconda.id.ovsagent.passwd:
            entry1.set(anaconda.id.ovsagent.passwd)
            entry2.set(anaconda.id.ovsagent.passwd)
        else:
            entry1.set("")
            entry2.set("")
        
        bb = ButtonBar (screen, (TEXT_OK_BUTTON, TEXT_BACK_BUTTON))

        thegrid.setField (Label (_("Password:")), 0, 0, (0, 0, 1, 0), anchorLeft = 1)
        thegrid.setField (Label (_("Password (confirm):")), 0, 1, (0, 0, 1, 0), anchorLeft = 1)
        thegrid.setField (entry1, 1, 0)
        thegrid.setField (entry2, 1, 1)
        toplevel.add (thegrid, 0, 1, (0, 0, 0, 1), anchorLeft = 1)
        toplevel.add (bb, 0, 2, growx = 1)

        while 1:
            toplevel.setCurrent(entry1)
            result = toplevel.run()
            rc = bb.buttonPressed (result)

            if rc == TEXT_BACK_CHECK:
                screen.popWindow()
                return INSTALL_BACK

            if len (entry1.value ()) < 6:
                handleShortPasswords(screen)
            elif entry1.value () != entry2.value ():
                handleMismatchingPasswords(screen)
            elif has_bad_chars(entry1.value()):
                handleBadCharacters(screen)
            else:
                break

            entry1.set('')
            entry2.set('')

        screen.popWindow()        
        anaconda.id.ovsagent.passwd = entry1.value()
        return INSTALL_OK

# vim:tw=78:ts=4:et:sw=4
